@@ -904,8 +904,8 @@ gettimeofday(struct timeval* tp, void* tzp)
   t = ((ft.dwHighDateTime * FOUR_GIGA_NSEC100) - EPOCH_WIN_UNIX_DELTA) +
       (ft.dwLowDateTime  * NSEC100);
 
-  tv->tv_sec  = (long) t;
-  tv->tv_usec = (long) ((t - tv->tv_sec) * 1e6);
+  tp->tv_sec  = (long) t;
+  tp->tv_usec = (long) ((t - tp->tv_sec) * 1e6);
 
   /* tzp is ignored */
 
@@ -2318,7 +2318,7 @@ flickcurl_photos_list_params_init(flickcurl_photos_list_params* list_params)
   if(!list_params)
     return 1;
   
-  memset(list_params, '\0', sizeof(list_params));
+  memset(list_params, '\0', sizeof(*list_params));
   list_params->version = 1;
 
   list_params->extras = NULL;
