@@ -2,7 +2,7 @@
  *
  * flickcurl.h - Flickcurl API
  *
- * Copyright (C) 2007-2010, David Beckett http://www.dajobe.org/
+ * Copyright (C) 2007-2013, David Beckett http://www.dajobe.org/
  * 
  * This file is licensed under the following three licenses as alternatives:
  *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
@@ -912,9 +912,9 @@ typedef struct {
  * @name: Group Name
  * @description: Description
  * @lang: Language
- * @is_admin: is admin flag
+ * @is_admin: user is a group admin
  * @is_pool_moderated: is the pool moderated
- * @is_eighteenplus: 18+ group
+ * @is_eighteenplus: NOT USED (will always be 0)
  * @privacy: privacy level
  * @photos: photos in group count
  * @iconserver: icon server ID
@@ -922,6 +922,21 @@ typedef struct {
  * @throttle_count: throttle count
  * @throttle_mode: throttle mode (day, ...)
  * @throttle_remaining: throttle remaining
+ * @iconfarm: icon farm ID
+ * @is_moderator: user is a moderator
+ * @is_member: user is a member
+ * @rules: group rules
+ * @pool_count: pool count
+ * @topic_count: topic count
+ * @photos_ok: photos are ok
+ * @videos_ok: videos are ok
+ * @images_ok: images are ok
+ * @screens_ok: screens are ok
+ * @art_ok: art is ok
+ * @safe_ok: safe content is ok
+ * @moderate_ok: moderate content is ok
+ * @restricted_ok: restricted content is ok
+ * @has_geo: has geolocation
  *
  * A group.
  */
@@ -940,6 +955,21 @@ typedef struct {
   int throttle_count;
   char* throttle_mode;
   int throttle_remaining;
+  int iconfarm;
+  int is_moderator;
+  int is_member;
+  char* rules;
+  int pool_count;
+  int topic_count;
+  int photos_ok;
+  int videos_ok;
+  int images_ok;
+  int screens_ok;
+  int art_ok;
+  int safe_ok;
+  int moderate_ok;
+  int restricted_ok;
+  int has_geo;
 } flickcurl_group;
 
 
@@ -1401,6 +1431,22 @@ typedef struct {
  *
  * A photo at a size.
  *
+ * Supported sizes and their letter suffixes used in the @source and
+ * the @url fields are as follows:
+ *
+ * s: small square 75x75.
+ * q: large square 150x150 (2011-03-12 or later).
+ * t: thumbnail, 100 on longest side.
+ * m: small, 240 on longest side.
+ * n: small, 320 on longest side (2011-03-12 or later).
+ * -: medium, 500 on longest side.
+ * z: medium 640, 640 on longest side.
+ * b: large, 1024 on longest side*.
+ * o: original image, either a jpg, gif or png, depending on source format.
+ *
+ * The 'q' and 'n' sizes were announced 2011-03-12 in
+ * http://tech.groups.yahoo.com/group/yws-flickr/message/7483
+ *
  */
 typedef struct {
   char *label;
@@ -1840,7 +1886,43 @@ FLICKCURL_API
 char* flickcurl_auth_getToken(flickcurl* fc, const char* frob);
 FLICKCURL_API
 int flickcurl_auth_oauth_getAccessToken(flickcurl* fc);
-  
+FLICKCURL_API
+const char* flickcurl_get_oauth_token(flickcurl *fc);
+FLICKCURL_API
+void flickcurl_set_oauth_token(flickcurl *fc, const char* token);
+FLICKCURL_API
+const char* flickcurl_get_oauth_token_secret(flickcurl* fc);
+FLICKCURL_API
+void flickcurl_set_oauth_token_secret(flickcurl* fc, const char *secret);
+FLICKCURL_API
+const char* flickcurl_get_oauth_client_key(flickcurl *fc);
+FLICKCURL_API
+const char* flickcurl_get_oauth_client_secret(flickcurl *fc);
+FLICKCURL_API
+void flickcurl_set_oauth_client_key(flickcurl *fc, const char* client_key);
+FLICKCURL_API
+void flickcurl_set_oauth_client_secret(flickcurl *fc, const char* client_secret);
+FLICKCURL_API
+const char* flickcurl_get_oauth_request_token(flickcurl* fc);
+FLICKCURL_API
+const char* flickcurl_get_oauth_request_token_secret(flickcurl* fc);
+FLICKCURL_API
+void flickcurl_set_oauth_request_token(flickcurl *fc, const char* token);
+FLICKCURL_API
+void flickcurl_set_oauth_request_token_secret(flickcurl *fc, const char* secret);
+FLICKCURL_API
+const char* flickcurl_get_oauth_username(flickcurl* fc);
+FLICKCURL_API
+const char* flickcurl_get_oauth_user_nsid(flickcurl* fc);
+
+/* OAuth flow requests */
+FLICKCURL_API
+int flickcurl_oauth_create_request_token(flickcurl* fc, const char* callback);
+FLICKCURL_API
+char* flickcurl_oauth_get_authorize_uri(flickcurl* fc);
+FLICKCURL_API
+int flickcurl_oauth_create_access_token(flickcurl* fc, const char* verifier);
+
 /* flickr.blogs */
 FLICKCURL_API
 flickcurl_blog** flickcurl_blogs_getList(flickcurl* fc);
@@ -2022,6 +2104,8 @@ flickcurl_exif** flickcurl_photos_getExif(flickcurl* fc, const char* photo_id, c
 FLICKCURL_API
 flickcurl_person** flickcurl_photos_getFavorites(flickcurl* fc, const char* photo_id, int page, int per_page);
 FLICKCURL_API
+flickcurl_photo* flickcurl_photos_getInfo2(flickcurl *fc, const char* photo_id, const char* secret);
+FLICKCURL_API FLICKCURL_DEPRECATED
 flickcurl_photo* flickcurl_photos_getInfo(flickcurl *fc, const char* photo_id);
 FLICKCURL_API
 flickcurl_photo** flickcurl_photos_getNotInSet(flickcurl* fc, int min_upload_date, int max_upload_date, const char* min_taken_date, const char* max_taken_date, int privacy_filter, const char* extras, int per_page, int page);
