@@ -169,6 +169,13 @@ flickcurl_oauth_free(flickcurl_oauth_data* od)
   
   if(od->data)
     free(od->data);
+
+  if(od->username)
+    free(od->username);
+  
+  if(od->user_nsid)
+    free(od->user_nsid);
+
 }
 
 
@@ -200,7 +207,7 @@ flickcurl_oauth_build_key(flickcurl_oauth_data* od)
   else
     od->key_len += od->token_secret_len;
 
-  od->key = malloc(od->key_len + 1); /* for NUL */
+  od->key = (unsigned char*)malloc(od->key_len + 1); /* for NUL */
   if(!od->key)
     return 1;
   
@@ -289,7 +296,6 @@ flickcurl_oauth_prepare_common(flickcurl *fc,
   char* timestamp = NULL;
   int rc = 0;
   int need_to_add_query = 0;
-  const char* http_method = "GET";
   int is_oauth_method = 0;
   char *p;
 
@@ -455,6 +461,7 @@ flickcurl_oauth_prepare_common(flickcurl *fc,
     size_t param_buf_len = 0;
     size_t vlen = 0;
     char *escaped_value = NULL;
+    const char* http_method;
     size_t http_method_len;
     size_t escaped_value_len;
     
@@ -482,7 +489,8 @@ flickcurl_oauth_prepare_common(flickcurl *fc,
     }
     *p = '\0';
 
-    http_method_len = strlen(http_method);
+    http_method = (upload_field || fc->is_write) ? "POST" : "GET";
+    http_method_len = (upload_field || fc->is_write) ? 4  : 3;
 
     buf_len = http_method_len;
     buf_len += 1; /* & */
@@ -798,6 +806,8 @@ flickcurl_oauth_create_access_token(flickcurl* fc, const char* verifier)
   flickcurl_oauth_data* od = &fc->od;
   char* access_token = NULL;
   char* access_token_secret = NULL;
+  char* username = NULL;
+  char* user_nsid = NULL;
   char** form = NULL;
   int rc = 0;
   const char* uri = fc->oauth_access_token_uri;
@@ -844,8 +854,12 @@ flickcurl_oauth_create_access_token(flickcurl* fc, const char* verifier)
       access_token = form[i+1];
     } else if(!strcmp(form[i], "oauth_token_secret")) {
       access_token_secret = form[i+1];
+    } else if(!strcmp(form[i], "username")) {
+      username = form[i+1];
+    } else if(!strcmp(form[i], "user_nsid")) {
+      user_nsid = form[i+1];
     }
-    /* ignoring: fullname, user_nsid, username */
+    /* ignoring: fullname */
   }
 
   if(access_token && access_token_secret) {
@@ -860,6 +874,26 @@ flickcurl_oauth_create_access_token(flickcurl* fc, const char* verifier)
     memcpy(od->token_secret, access_token_secret, len + 1);
     od->token_secret_len = len;
 
+    if(username) {
+      len = strlen(username);
+      od->username = (char*)malloc(len + 1);
+      memcpy(od->username, username, len + 1);
+      od->username_len = len;
+    } else {
+      od->username = NULL;
+      od->username_len = 0;
+    }
+
+    if(user_nsid) {
+      len = strlen(user_nsid);
+      od->user_nsid = (char*)malloc(len + 1);
+      memcpy(od->user_nsid, user_nsid, len + 1);
+      od->user_nsid_len = len;
+    } else {
+      od->user_nsid = NULL;
+      od->user_nsid_len = 0;
+    }
+    
     /* Delete temporary request token and secret */
     free(od->request_token);
     od->request_token = NULL;
@@ -1030,6 +1064,33 @@ flickcurl_get_oauth_token_secret(flickcurl* fc)
   return fc->od.token_secret;
 }
 
+/**
+ * flickcurl_get_oauth_username:
+ * @fc: flickcurl object
+ *
+ * Get the username for the authenticated user
+ *
+ * Return value: username or NULL if none set
+ */
+const char*
+flickcurl_get_oauth_username(flickcurl* fc)
+{
+  return fc->od.username;
+}
+
+/**
+ * flickcurl_get_oauth_user_nsid:
+ * @fc: flickcurl object
+ *
+ * Get the user_nsid for the authenticated user
+ *
+ * Return value: user_nsid or NULL if none set
+ */
+const char*
+flickcurl_get_oauth_user_nsid(flickcurl* fc)
+{
+  return fc->od.user_nsid;
+}
 
 /**
  * flickcurl_set_oauth_token_secret:
