@@ -2,7 +2,7 @@
  *
  * flickr.panda-api.c - Flickr flickr.panda.* API calls
  *
- * Copyright (C) 2009, David Beckett http://www.dajobe.org/
+ * Copyright (C) 2009-2012, David Beckett http://www.dajobe.org/
  * 
  * This file is licensed under the following three licenses as alternatives:
  *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
@@ -77,8 +77,9 @@ flickcurl_build_pandas(flickcurl* fc,
     /* Use first text child node of <panda> as panda name */
     for(chnode = node->children; chnode; chnode = chnode->next) {
       if(chnode->type  == XML_TEXT_NODE) {
-        panda = (char*)malloc(strlen((const char*)chnode->content)+1);
-        strcpy(panda, (const char*)chnode->content);
+        size_t len = strlen((const char*)chnode->content);
+        panda = (char*)malloc(len + 1);
+        memcpy(panda, chnode->content, len + 1);
         break;
       }
     }
@@ -121,15 +122,15 @@ flickcurl_build_pandas(flickcurl* fc,
 char**
 flickcurl_panda_getList(flickcurl* fc)
 {
-  const char* parameters[7][2];
-  int count = 0;
   xmlDocPtr doc = NULL;
   xmlXPathContextPtr xpathCtx = NULL; 
   char **pandas = NULL;
   
-  parameters[count][0]  = NULL;
+  flickcurl_init_params(fc);
 
-  if(flickcurl_prepare(fc, "flickr.panda.getList", parameters, count))
+  flickcurl_end_params(fc);
+
+  if(flickcurl_prepare(fc, "flickr.panda.getList"))
     goto tidy;
 
   doc = flickcurl_invoke(fc);
@@ -153,8 +154,11 @@ flickcurl_panda_getList(flickcurl* fc)
   if(xpathCtx)
     xmlXPathFreeContext(xpathCtx);
 
-  if(fc->failed)
+  if(fc->failed) {
+    if(pandas)
+      free(pandas);
     pandas = NULL;
+  }
 
   return pandas;
 }
@@ -185,21 +189,20 @@ flickcurl_panda_getList(flickcurl* fc)
 flickcurl_photo**
 flickcurl_panda_getPhotos(flickcurl *fc, const char *panda_name)
 {
-  const char* parameters[8][2];
-  int count = 0;
   flickcurl_photo** photos = NULL;
   flickcurl_photos_list* photos_list = NULL;
   const char* format = NULL;
   
+  flickcurl_init_params(fc);
+
   if(!panda_name)
     return NULL;
 
-  parameters[count][0]  = "panda_name";
-  parameters[count++][1]= panda_name;
+  flickcurl_add_param(fc, "panda_name", panda_name);
 
-  parameters[count][0]  = NULL;
+  flickcurl_end_params(fc);
 
-  if(flickcurl_prepare(fc, "flickr.panda.getPhotos", parameters, count))
+  if(flickcurl_prepare(fc, "flickr.panda.getPhotos"))
     goto tidy;
 
   photos_list = flickcurl_invoke_photos_list(fc,
