@@ -2,7 +2,7 @@
  *
  * reflection-api.c - Flickr flickr.reflection.* API calls
  *
- * Copyright (C) 2007-2008, David Beckett http://www.dajobe.org/
+ * Copyright (C) 2007-2012, David Beckett http://www.dajobe.org/
  * 
  * This file is licensed under the following three licenses as alternatives:
  *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
@@ -55,8 +55,6 @@
 char**
 flickcurl_reflection_getMethods(flickcurl* fc)
 {
-  const char * parameters[5][2];
-  int count = 0;
   xmlDocPtr doc = NULL;
   xmlXPathContextPtr xpathCtx = NULL; 
   xmlXPathObjectPtr xpathObj = NULL;
@@ -65,10 +63,13 @@ flickcurl_reflection_getMethods(flickcurl* fc)
   int i;
   int size;
   char **methods = NULL;
+  int count;
   
-  parameters[count][0]  = NULL;
+  flickcurl_init_params(fc);
 
-  if(flickcurl_prepare(fc, "flickr.reflection.getMethods", parameters, count))
+  flickcurl_end_params(fc);
+
+  if(flickcurl_prepare_noauth(fc, "flickr.reflection.getMethods"))
     goto tidy;
 
   doc = flickcurl_invoke(fc);
@@ -109,8 +110,9 @@ flickcurl_reflection_getMethods(flickcurl* fc)
     /* Walk children nodes for description text */
     for(chnode = node->children; chnode; chnode = chnode->next) {
       if(chnode->type == XML_TEXT_NODE) {
-        methods[count] = (char*)malloc(strlen((const char*)chnode->content)+1);
-        strcpy(methods[count], (const char*)chnode->content);
+        size_t len = strlen((const char*)chnode->content);
+        methods[count] = (char*)malloc(len + 1);
+        memcpy(methods[count], chnode->content, len + 1);
         count++;
         break;
       }
@@ -144,18 +146,17 @@ flickcurl_reflection_getMethods(flickcurl* fc)
 flickcurl_method*
 flickcurl_reflection_getMethodInfo(flickcurl* fc, const char* name)
 {
-  const char * parameters[6][2];
-  int count = 0;
   xmlDocPtr doc = NULL;
   xmlXPathContextPtr xpathCtx = NULL; 
   flickcurl_method* method = NULL;
   
-  parameters[count][0]  = "method_name";
-  parameters[count++][1]= name;
+  flickcurl_init_params(fc);
 
-  parameters[count][0]  = NULL;
+  flickcurl_add_param(fc, "method_name", name);
 
-  if(flickcurl_prepare(fc, "flickr.reflection.getMethodInfo", parameters, count))
+  flickcurl_end_params(fc);
+
+  if(flickcurl_prepare(fc, "flickr.reflection.getMethodInfo"))
     goto tidy;
 
   doc = flickcurl_invoke(fc);
